@@ -1,4 +1,4 @@
-`logLik.weibull.frailty` <-
+logLik.weibull.frailty <-
 function (object, ...) {
     out <- object$logLik
     attr(out, "df") <- length(unlist(object$coefficients))
