@@ -17,7 +17,13 @@ function (x, y, id, initial.values, parameterization, derivForm, control) {
     Ztime.deriv <- x$Ztime.deriv
     Zs.deriv <- x$Zs.deriv
     WW <- x$W
+    WintF.vl <- x$WintF.vl
+    WintF.sl <- x$WintF.sl
+    Ws.intF.vl <- x$Ws.intF.vl
+    Ws.intF.sl <- x$Ws.intF.sl
     X <- dropAttr(X); Z <- dropAttr(Z); WW <- dropAttr(WW)
+    WintF.vl <- dropAttr(WintF.vl); WintF.sl <- dropAttr(WintF.sl)
+    Ws.intF.vl <- dropAttr(Ws.intF.vl); Ws.intF.sl <- dropAttr(Ws.intF.sl)
     if (parameterization == "value") {
         Xtime <- dropAttr(Xtime); Ztime <- dropAttr(Ztime); Xs <- dropAttr(Xs); Zs <- dropAttr(Zs)
     } else if (parameterization == "slope") {
@@ -46,11 +52,18 @@ function (x, y, id, initial.values, parameterization, derivForm, control) {
     outer.Ztime <- lapply(1:n, function (x) Ztime[x, ] %o% Ztime[x, ])
     # Gauss-Hermite quadrature rule components
     GH <- gauher(control$GHk)
-    b <- as.matrix(expand.grid(lapply(1:ncz, function (k, u) u$x, u = GH)))
+    b <- as.matrix(expand.grid(rep(list(GH$x), ncz)))
     k <- nrow(b)
-    wGH <- as.matrix(expand.grid(lapply(1:ncz, function (k, u) u$w, u = GH)))
-    wGH <- 2^(ncz/2) * apply(wGH, 1, prod) * exp(rowSums(b * b)) * control$det.inv.chol.VC
-    b <- sqrt(2) * t(control$inv.chol.VC %*% t(b)); dimnames(b) <- NULL
+    wGH <- as.matrix(expand.grid(rep(list(GH$w), ncz)))    
+    wGH <- 2^(ncz/2) * apply(wGH, 1, prod) * exp(rowSums(b * b))
+    if (control$typeGH == "simple") {
+        b <- sqrt(2) * t(control$inv.chol.VC %*% t(b))
+        wGH <- wGH * control$det.inv.chol.VC
+    } else { 
+        b <- sqrt(2) * b
+       VCdets <- control$det.inv.chol.VCs
+    }
+    dimnames(b) <- NULL
     b2 <- if (ncz == 1) b * b else t(apply(b, 1, function (x) x %o% x))
     Ztb <- Z %*% t(b)
     if (parameterization %in% c("value", "both")) {
@@ -73,10 +86,31 @@ function (x, y, id, initial.values, parameterization, derivForm, control) {
     ind.K <- rep(unlist(lapply(ind.D, seq_len)), each = nk)
     wk <- unlist(lapply(ind.D, function (n) rep(x$wk, n)))
     wkP <- wk * rep(x$P, each = nk)
+    # pseudo-adaptive Gauss-Hermite
+    if (control$typeGH != "simple") {
+        lis.b <- vector("list", n)
+        for (i in 1:n)
+            lis.b[[i]] <- t(control$inv.chol.VCs[[i]] %*% t(b)) + 
+                rep(control$ranef[i, ], each = k)
+        lis.b2 <- lapply(lis.b, function (b) if (ncz == 1) b * b else
+            t(apply(b, 1, function (x) x %o% x)))
+        for (i in 1:n) {
+            Ztb[id == i, ] <- Z[id == i, , drop = FALSE] %*% t(lis.b[[i]])
+            if (parameterization %in% c("value", "both")) {
+                Ztime.b[i, ] <- Ztime[i, , drop = FALSE] %*% t(lis.b[[i]])
+                Zsb[id.GK == i, ] <- Zs[id.GK == i, ] %*% t(lis.b[[i]])
+            }
+            if (parameterization %in% c("slope", "both") && 
+                    (length(indRandom) > 1 || indRandom)) {
+                Ztime.b.deriv[i, ] <- Ztime.deriv[i, , drop = FALSE] %*% t(lis.b[[i]][, indRandom, drop = FALSE])
+                Zsb.deriv[id.GK == i, ] <- Zs.deriv[id.GK == i, ] %*% t(lis.b[[i]][, indRandom, drop = FALSE])
+            }            
+        }
+    }
     # initial values
     betas <- as.vector(initial.values$betas)
     sigma <- initial.values$sigma
-    gammas <- as.vector(initial.values$gammas)
+    gammas <- if (!is.null(WW)) as.vector(initial.values$gammas) else NULL
     alpha <- as.vector(initial.values$alpha)
     Dalpha <- as.vector(initial.values$Dalpha)
     xi <- initial.values$xi
@@ -92,15 +126,18 @@ function (x, y, id, initial.values, parameterization, derivForm, control) {
     # EM iterations
     iter <- control$iter.EM
     Y.mat <- matrix(0, iter + 1, ncx + 1)
-    T.mat <- matrix(0, iter + 1, switch(parameterization, "value" = , "slope" = ncww + length(xi) + 1, "both" = ncww + length(xi) + 2))
+    T.mat <- matrix(0, iter + 1, switch(parameterization, 
+        "value" = ncww + length(xi) + ncol(WintF.vl), 
+        "slope" = ncww + length(xi) + ncol(WintF.sl), 
+        "both" = ncww + length(xi) + ncol(WintF.vl) + ncol(WintF.sl)))
     B.mat <- if (diag.D) matrix(0, iter + 1, ncz) else matrix(0, iter + 1, ncz * ncz)
     lgLik <- numeric(iter + 1)
-    conv <- FALSE
+    conv <- TRUE
     for (it in 1:iter) {
         # save parameter values in matrix
         Y.mat[it, ] <- c(betas, sigma)
-        T.mat[it, ] <- switch(parameterization, "value" = c(gammas, alpha, xi), 
-            "slope" = c(gammas, Dalpha, xi), "both" = c(gammas, alpha, Dalpha, xi))
+        T.mat[it, ] <- switch(parameterization, "value" = c(gammas, alpha, log(xi)), 
+            "slope" = c(gammas, Dalpha, log(xi)), "both" = c(gammas, alpha, Dalpha, log(xi)))
         B.mat[it,] <- D
         
         # linear predictors
@@ -109,14 +146,20 @@ function (x, y, id, initial.values, parameterization, derivForm, control) {
         if (parameterization %in% c("value", "both")) {
             Y <- as.vector(Xtime %*% betas) + Ztime.b
             Ys <- as.vector(Xs %*% betas) + Zsb
-            eta.t <- eta.tw + alpha * Y
-            eta.s <- alpha * Ys
+            eta.t <- eta.tw + c(WintF.vl %*% alpha) * Y
+            eta.s <- c(Ws.intF.vl %*% alpha) * Ys
         }
         if (parameterization %in% c("slope", "both")) {
             Y.deriv <- as.vector(Xtime.deriv %*% betas[indFixed]) + Ztime.b.deriv
             Ys.deriv <- as.vector(Xs.deriv %*% betas[indFixed]) + Zsb.deriv
-            eta.t <- if (parameterization == "both") eta.t + Dalpha * Y.deriv else eta.tw + Dalpha * Y.deriv
-            eta.s <- if (parameterization == "both") eta.s + Dalpha * Ys.deriv else Dalpha * Ys.deriv
+            eta.t <- if (parameterization == "both")
+                eta.t + c(WintF.sl %*% Dalpha) * Y.deriv
+            else
+                eta.tw + c(WintF.sl %*% Dalpha) * Y.deriv
+            eta.s <- if (parameterization == "both")
+                eta.s + c(Ws.intF.sl %*% Dalpha) * Ys.deriv 
+            else
+                c(Ws.intF.sl %*% Dalpha) * Ys.deriv
         }
         
         # E-step
@@ -127,25 +170,35 @@ function (x, y, id, initial.values, parameterization, derivForm, control) {
         log.survival <- - exp(eta.tw) * rowsum(xi[ind.K] * wkP * exp(eta.s), id.GK, reorder = FALSE)
         dimnames(log.survival) <- NULL
         log.p.tb <- d * log.hazard + log.survival
-        log.p.b <- if (ncz == 1) {
-            dnorm(b, sd = sqrt(D), log = TRUE)
+        log.p.b <- if (control$typeGH == "simple") {
+            rep(dmvnorm(b, rep(0, ncz), D, TRUE), each = n)
         } else {
-            if (diag.D) {
-                rowSums(dnorm(b, sd = rep(sqrt(D), each = k), log = TRUE))
-            } else {
-                dmvnorm(b, rep(0, ncz), D, TRUE)
-            }
+            matrix(dmvnorm(do.call(rbind, lis.b), rep(0, ncz), D, TRUE), n, k, byrow = TRUE)
         }
-        p.ytb <- exp((log.p.yb + log.p.tb) + rep(log.p.b, each = n))
+        p.ytb <- exp(log.p.yb + log.p.tb + log.p.b)
+        if (control$typeGH != "simple")
+            p.ytb <- p.ytb * VCdets
         p.yt <- c(p.ytb %*% wGH)
         p.byt <- p.ytb / p.yt
-        post.b <- p.byt %*% (b * wGH)
-        post.vb <- if (ncz == 1) {
-            c(p.byt %*% (b2 * wGH)) - c(post.b * post.b)
+        post.b <- if (control$typeGH == "simple") {
+            p.byt %*% (b * wGH)
         } else {
-            (p.byt %*% (b2 * wGH)) - t(apply(post.b, 1, function (x) x %o% x))
+            sapply(seq_len(ncz), function (i)
+                (p.byt * t(sapply(lis.b, "[", seq_len(k), i))) %*% wGH)
         }
-        
+        post.vb <- if (control$typeGH == "simple") {
+            if (ncz == 1) {
+                c(p.byt %*% (b2 * wGH)) - c(post.b * post.b)
+            } else {
+                (p.byt %*% (b2 * wGH)) - t(apply(post.b, 1, function (x) x %o% x))
+            }
+        } else {
+            dd <- sapply(seq_len(ncz^2), function (i)
+                (p.byt * t(sapply(lis.b2, "[", seq_len(k), i))) %*% wGH)
+            bb <- apply(post.b, 1, function (x) x %o% x)
+            dd - if (ncz == 1) c(bb) else t(bb)
+        }
+
         # compute log-likelihood
         log.p.yt <- log(p.yt)
         lgLik[it] <- sum(log.p.yt[is.finite(log.p.yt)], na.rm = TRUE)
@@ -157,8 +210,8 @@ function (x, y, id, initial.values, parameterization, derivForm, control) {
             cat("betas:", round(betas, 4), "\n")
             cat("sigma:", round(sigma, 4), "\n")
             if (!is.null(WW)) cat("gammas:", round(gammas, 4), "\n")
-            if (parameterization %in% c("value", "both")) cat("alpha:", -round(alpha, 4), "\n")
-            if (parameterization %in% c("slope", "both")) cat("alphaD:", -round(Dalpha, 4), "\n")
+            if (parameterization %in% c("value", "both")) cat("alpha:", round(alpha, 4), "\n")
+            if (parameterization %in% c("slope", "both")) cat("alphaD:", round(Dalpha, 4), "\n")
             cat("xi:", round(xi, 4), "\n")
             cat("D:", if (!diag.D) round(D[lower.tri(D, TRUE)], 4) else round(D, 4), "\n")
         }
@@ -170,7 +223,7 @@ function (x, y, id, initial.values, parameterization, derivForm, control) {
             check1 <- max(abs(thets2 - thets1) / (abs(thets1) + control$tol1)) < control$tol2
             check2 <- (lgLik[it] - lgLik[it - 1]) < control$tol3 * (abs(lgLik[it - 1]) + control$tol3)
             if (check1 || check2) {
-                conv <- TRUE
+                conv <- FALSE
                 if (control$verbose)
                     cat("\n\nconverged!\n")
                 break
@@ -183,7 +236,11 @@ function (x, y, id, initial.values, parameterization, derivForm, control) {
         mu <- y - eta.yx
         tr.tZZvarb <- sum(ZtZ * post.vb, na.rm = TRUE)
         sigman <- sqrt(c(crossprod(mu, mu - 2 * Zb) + crossprod(Zb) + tr.tZZvarb) / N)
-        Dn <- matrix(colMeans(p.byt %*% (b2 * wGH), na.rm = TRUE), ncz, ncz)
+        Dn <- if (control$typeGH == "simple") {
+            matrix(colMeans(p.byt %*% (b2 * wGH), na.rm = TRUE), ncz, ncz)
+        } else {
+            matrix(colMeans(dd, na.rm = TRUE), ncz, ncz)
+        }
         Dn <- if (diag.D) diag(Dn) else 0.5 * (Dn + t(Dn))
         Hbetas <- nearPD(H.longPC(betas))
         scbetas <- gr.longPC(betas)
@@ -203,15 +260,16 @@ function (x, y, id, initial.values, parameterization, derivForm, control) {
         gammas <- thetasn$gammas
         alpha <- thetasn$alpha
         Dalpha <- thetasn$Dalpha
-        xi <- exp(thetasn$log.xi)
+        xi <- exp(thetasn$log.xi)        
     }
-    list.thetas <- list(betas = betas, log.sigma = log(sigma), gammas = gammas, alpha = alpha, Dalpha = Dalpha,
-        log.xi = log(xi), D = if (diag.D) log(D) else chol.transf(D))    
+    list.thetas <- list(betas = betas, log.sigma = log(sigma), gammas = gammas, 
+        alpha = alpha, Dalpha = Dalpha, log.xi = log(xi), 
+        D = if (diag.D) log(D) else chol.transf(D))    
     list.thetas <- list.thetas[!sapply(list.thetas, is.null)]
     thetas <- unlist(as.relistable(list.thetas))
     lgLik <- - LogLik.piecewiseGH(thetas)
     # if not converged, start quasi-Newton iterations
-    if (!conv && !control$only.EM) {
+    if (conv && !control$only.EM) {
         if (is.null(control$parscale))
             control$parscale <- rep(0.01, length(thetas))
         if (control$verbose)
@@ -242,14 +300,20 @@ function (x, y, id, initial.values, parameterization, derivForm, control) {
             if (parameterization %in% c("value", "both")) {
                 Y <- as.vector(Xtime %*% betas) + Ztime.b
                 Ys <- as.vector(Xs %*% betas) + Zsb
-                eta.t <- eta.tw + alpha * Y
-                eta.s <- alpha * Ys
+                eta.t <- eta.tw + c(WintF.vl %*% alpha) * Y
+                eta.s <- c(Ws.intF.vl %*% alpha) * Ys
             }
             if (parameterization %in% c("slope", "both")) {
                 Y.deriv <- as.vector(Xtime.deriv %*% betas[indFixed]) + Ztime.b.deriv
                 Ys.deriv <- as.vector(Xs.deriv %*% betas[indFixed]) + Zsb.deriv
-                eta.t <- if (parameterization == "both") eta.t + Dalpha * Y.deriv else eta.tw + Dalpha * Y.deriv
-                eta.s <- if (parameterization == "both") eta.s + Dalpha * Ys.deriv else Dalpha * Ys.deriv
+                eta.t <- if (parameterization == "both")
+                    eta.t + c(WintF.sl %*% Dalpha) * Y.deriv
+                else
+                    eta.tw + c(WintF.sl %*% Dalpha) * Y.deriv
+                eta.s <- if (parameterization == "both")
+                    eta.s + c(Ws.intF.sl %*% Dalpha) * Ys.deriv 
+                else
+                    c(Ws.intF.sl %*% Dalpha) * Ys.deriv
             }
             exp.eta.tw <- exp(eta.tw)
             mu.y <- eta.yx + Ztb
@@ -259,24 +323,33 @@ function (x, y, id, initial.values, parameterization, derivForm, control) {
             log.survival <- - exp(eta.tw) * rowsum(xi[ind.K] * wkP * exp(eta.s), id.GK, reorder = FALSE)
             dimnames(log.survival) <- NULL
             log.p.tb <- d * log.hazard + log.survival            
-            log.p.b <- if (ncz == 1) {
-                dnorm(b, sd = sqrt(D), log = TRUE)
+            log.p.b <- if (control$typeGH == "simple") {
+                rep(dmvnorm(b, rep(0, ncz), D, TRUE), each = n)
             } else {
-                if (diag.D) {
-                    rowSums(dnorm(b, sd = rep(sqrt(D), each = k), log = TRUE))
-                } else {
-                    dmvnorm(b, rep(0, ncz), D, TRUE)
-                }
+                matrix(dmvnorm(do.call(rbind, lis.b), rep(0, ncz), D, TRUE), n, k, byrow = TRUE)
             }
-            p.ytb <- exp((log.p.yb + log.p.tb) + rep(log.p.b, each = n))
-            dimnames(p.ytb) <- NULL
+            p.ytb <- exp(log.p.yb + log.p.tb + log.p.b)
+            if (control$typeGH != "simple")
+                p.ytb <- p.ytb * VCdets
             p.yt <- c(p.ytb %*% wGH)
             p.byt <- p.ytb / p.yt
-            post.b <- p.byt %*% (b * wGH)
-            post.vb <- if (ncz == 1) {
-                c(p.byt %*% (b2 * wGH)) - c(post.b * post.b)
+            post.b <- if (control$typeGH == "simple") {
+                p.byt %*% (b * wGH)
             } else {
-                (p.byt %*% (b2 * wGH)) - t(apply(post.b, 1, function (x) x %o% x))
+                sapply(seq_len(ncz), function (i)
+                    (p.byt * t(sapply(lis.b, "[", seq_len(k), i))) %*% wGH)
+            }
+            post.vb <- if (control$typeGH == "simple") { 
+                if (ncz == 1) {
+                    c(p.byt %*% (b2 * wGH)) - c(post.b * post.b)
+                } else {
+                    (p.byt %*% (b2 * wGH)) - t(apply(post.b, 1, function (x) x %o% x))
+                }
+            } else {
+                dd <- sapply(seq_len(ncz^2), function (i)
+                    (p.byt * t(sapply(lis.b2, "[", seq_len(k), i))) %*% wGH)
+                bb <- apply(post.b, 1, function (x) x %o% x)
+                dd - if (ncz == 1) c(bb) else t(bb)
             }
             Zb <- if (ncz == 1) post.b[id] else rowSums(Z * post.b[id, ], na.rm = TRUE)
         }
@@ -292,7 +365,29 @@ function (x, y, id, initial.values, parameterization, derivForm, control) {
     if (!diag.D) dimnames(D) <- dimnames(initial.values$D) else names(D) <- names(initial.values$D)
     if (!is.null(WW))
         names(gammas) <- colnames(x$W)
-    gg <- switch(parameterization, "value" = "alpha", "slope" = "alphaD", "both" = c("alpha", "alphaD"))
+    nm.alph <- colnames(x$WintF.vl)
+    nm.alph <- if (!is.null(nm.alph)) {
+        if (nm.alph[1] == "(Intercept)")
+            c("", nm.alph[-1])
+        else
+            nm.alph
+    } else {
+        "alpha"
+    }
+    nm.Dalph <- colnames(x$WintF.sl)
+    nm.Dalph <- if (!is.null(nm.Dalph)) {
+        if (nm.Dalph[1] == "(Intercept)")
+            c("", nm.Dalph[-1])
+        else
+            nm.Dalph
+    } else {
+        "alpha.s"
+    }
+    gg <- switch(parameterization, "value" = nm.alph, "slope" = nm.Dalph, "both" = c(nm.alph, nm.Dalph))
+    if (parameterization %in% c("value", "both"))
+        names(alpha) <- nm.alph
+    if (parameterization %in% c("slope", "both"))
+        names(Dalpha) <- nm.Dalph
     names(xi) <- paste("xi.", seq_len(Q), sep = "")
     nams <- c(paste("Y.", c(names(betas), "sigma"), sep = ""), paste("T.", c(names(gammas), gg, names(xi)), sep = ""),
         paste("B.", if (!diag.D) paste("D", seq(1, ncz * (ncz + 1) / 2), sep = "") else names(D), sep = ""))
